/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.*;
import java.lang.* ;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

class JdctFrame extends Frame  implements FocusListener
{
	private JdctRecordInstance recordInstance = null;

	private static Vector instanceVector = new Vector(10);
	private static JdctMenu menu;

	private GridBagLayout gbl;
	private GridBagConstraints gbc;

	private Label recordInstanceLabel;
	private Label filenameLabel;
        private Label activeWindowLabel;
	private Choice windowChoice;

	private int activeWindowNum;
	private java.awt.List typeList;
	private java.awt.List instanceList;
	private String selectedRecordName = null;
	private JdctRecordInstance selectedRecordInstance = null;

	JdctFrame(String[] filename) {
		super("Jdct");

		menu = new JdctMenu(this);
		setMenuBar(menu);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
			createQuitDialog();
			//new QuitDialog(JdctFrame.this,"Quit Dialog","Quit Jdct? . . . Note: If you have not saved your most recent changes, they will be lost");
			}
		}
		);

		//setBackground(SystemColor.control);
		setBackground(java.awt.Color.lightGray);

		Label recordTypeListLabel = new Label("Record types");
		Label instanceListLabel = new Label("Records");

		/*Add record type List*/
		typeList = new java.awt.List(6,true);

		CheckboxGroup cg = new CheckboxGroup();
		Checkbox allTypesCB = new Checkbox("all types",cg,true);
		allTypesCB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				int n = typeList.getItemCount();
				for(int i=0; i<n; i++) {
					typeList.select(i);
				}
			}
		}
		);
		Checkbox clearTypesCB = new Checkbox("clear types",cg,true);
		clearTypesCB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				int n = typeList.getItemCount();
				for(int i=0; i<n; i++) {
					typeList.deselect(i);
				}
			}
		}
		);

		Button getInstancesButton = new Button("Get instances");
		getInstancesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getRecordInstanceList();
			} 
		}
		);

		/*Add instanceList*/
		instanceList = new java.awt.List(6,false);
		instanceList.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				String name  = getRecordName();
				JdctRecordInstance instance;
				boolean load = true;
				for (int i=0; i<instanceVector.size(); i++) {
					instance = (JdctRecordInstance)(instanceVector.elementAt(i));
		       			 if ( instance.getRecordName().equals(name)) {
						recordInstance = instance;
						load = false;
						break;
					}
				}

				if ( name == null  || ( recordInstance != null && 
					name.equals(selectedRecordName) &&
					recordInstance.equals(selectedRecordInstance))) return;
				selectedRecordName  = name;
				selectedRecordInstance  = recordInstance;
				menu.setRecordName(selectedRecordName);
				if ( recordInstance == null ) {
					addRecordInstance();
				}
				if (load) recordInstance.loadInstance(selectedRecordName);
				windowChoice.select(recordInstance.getWindowName());
				activateRecordInstance(recordInstance);
			}
		}
		);

		/*Add selected window information*/
		filenameLabel = new Label("database filename: No filename");
		recordInstanceLabel = new Label("No instance loaded");
                activeWindowLabel =   new Label("Active Window is");

		/*Add window choices list and New Window button*/
		windowChoice = new Choice();
		windowChoice.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Choice ie = (Choice)e.getItemSelectable();
				int index = ie.getSelectedIndex();
				JdctRecordInstance instance = 
				    (JdctRecordInstance)instanceVector.
				    elementAt(index);
				activateRecordInstance(instance);
			}
		}
		);

		Button windowButton = new Button("New window");
		windowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addRecordInstance();
			} 
		}
		);


		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();

		setLayout(gbl);

		addComponent( 0,1,1,1,gbc.NONE,1,0,this,recordTypeListLabel);
		addComponent( 2,1,1,1,gbc.NONE,1,0,this,instanceListLabel);

		addComponent( 0,2,1,4,gbc.BOTH,10,10,this,typeList );
		addComponent( 2,2,1,4,gbc.BOTH,10,10,this,instanceList );

		addComponent( 1,2,1,1,gbc.HORIZONTAL,1,0,this,allTypesCB);
		addComponent( 1,3,1,1,gbc.HORIZONTAL,1,0,this,clearTypesCB );
		addComponent( 1,4,1,1,gbc.HORIZONTAL,1,0,this,getInstancesButton );

		addComponent( 0,6,3,1,gbc.HORIZONTAL,0,0,this,filenameLabel );

		addComponent( 0,7,3,1,gbc.HORIZONTAL,0,0,this,recordInstanceLabel );
                addComponent( 0,8,3,1,gbc.HORIZONTAL,0,0,this,activeWindowLabel);
		addComponent( 0,9,2,1,gbc.HORIZONTAL,0,0,this,windowChoice );
		addComponent( 2,9,1,1,gbc.HORIZONTAL,0,0,this,windowButton );

		setSize( 500,200 );

		//addRecordInstance();

		setLocation(25,20);
		pack();
		//show();

		for (int i=0; i<filename.length; i++) {
			if (filename[i].toLowerCase().endsWith(".db"))
				menu.openDbFile(filename[i]);
			else menu.openDbdFile(filename[i]);
		}
	}

	public DbBase getDbBase()
	    {
		return menu.getDbBase();
	}

	void removeRecordInstance(JdctRecordInstance recordInstanceIn)
	{
		if (!windowChoice.getSelectedItem().equals(
		    recordInstanceIn.getWindowName())) {
			windowChoice.select(0);
		}
		windowChoice.remove(recordInstanceIn.getWindowName());
		instanceVector.removeElement(recordInstanceIn);
		if (instanceVector.size() > 0) {
			int index = windowChoice.getSelectedIndex();
			JdctRecordInstance instance = 
			    (JdctRecordInstance)instanceVector.elementAt(index);
			activateRecordInstance(instance);
		}
		else {
			recordInstanceLabel.setText("No instance loaded");
			recordInstance=null;
		}
	}

	void addRecordInstance()
	{
		JdctRecordInstance instance = null;

		int windowId = instanceVector.size();
		for (int i=0; i<instanceVector.size(); i++) {
			instance = (JdctRecordInstance)(instanceVector.elementAt(i));
			if (instance.getWindowId() > i ){
				windowId = i;
				break;
			}
		}
		instance = new JdctRecordInstance(this,windowId);
		instanceVector.insertElementAt(instance,windowId);
		windowChoice.insert(instance.getWindowName(),windowId);
		windowChoice.select(windowId);
		activateRecordInstance(instance);
	}

	void clearAllRecordInstances()
	{
		JdctRecordInstance instance = null;

		getInstanceList().removeAll();
		int windowId = instanceVector.size();
		for (int i=0; i<instanceVector.size(); i++) {
			((JdctRecordInstance)(instanceVector.elementAt(i))).loadInstance(null);
		}
		activateRecordInstance(null);
	}

	void activateRecordInstance(JdctRecordInstance recordInstanceIn)
	{
		recordInstance = recordInstanceIn;
		//activeWindowNum = windowChoice.getSelectedIndex();
		activeWindowNum = recordInstance.getWindowId();
		if (recordInstanceIn != null) {
			recordInstanceIn.activate();
			setRecordInstance(recordInstanceIn.getTypeName(),
			    recordInstanceIn.getRecordName());
		} else {
			recordInstanceLabel.setText("No instance loaded");
			//setRecordInstance(" "," ");
			recordInstance=null;
		}
	}

	public JdctRecordInstance getRecordInstance()
	{
		return recordInstance;
	}

	public Vector getInstanceVector()
	{
		return instanceVector;
	}

	void setRecordInstance(String typeName, String recordName)
	{
		recordInstanceLabel.setText("Active record name: "+recordName + " in Active Window No. " + activeWindowNum);
                activeWindowLabel.setText("Select a different active window:");
	}

	public void getRecordInstanceList()
	    {
		int nTypes = getTypeList().getItemCount();
		if(nTypes<=0) return;
		String selectedTypes[] = getTypeList().getSelectedItems();
		DbBase dbBase = menu.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(getInstanceList().getItemCount()>0)
			getInstanceList().removeAll();
		for(int indType=0; indType<selectedTypes.length; indType++) {
			if(!dbEntry.findRecordType(selectedTypes[indType])) continue;
			if(!dbEntry.firstRecord()) continue;
			do {
				getInstanceList().add(dbEntry.getRecordName());
			} while (dbEntry.nextRecord());
		}
		dbEntry.free();
	}

	public void addComponent( int x, int y, int w, int h, int fill, int xWeight, int yWeight, Container aContainer, Component
	    aComponent )
	    {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		gbc.fill = fill;
		gbc.weightx = xWeight;
		gbc.weighty = yWeight;
		gbl.setConstraints( aComponent, gbc );
		aContainer.add( aComponent );
	}

	public void focusGained(FocusEvent e) {
		JdctInstanceGui gui = (JdctInstanceGui)e.getComponent();
		JdctRecordInstance newInstance = gui.getRecordInstance();
		windowChoice.select(newInstance.getWindowName());
		activateRecordInstance(newInstance);
	}

	public void focusLost(FocusEvent e) {
	}

	public String getRecordName() {
		return instanceList.getSelectedItem();
	}

	public java.awt.List getTypeList() {
		return typeList;
	}
	public java.awt.List getInstanceList() {
		return instanceList;
	}
	public void setFilename(String filename) {
		filenameLabel.setText(filename);
	}

	public void createQuitDialog()
	{
		new QuitDialog(JdctFrame.this,"Quit Dialog","Quit Jdct? . . . Note: If you have not saved your most recent changes, they will be lost");
	}

	class QuitDialog extends Dialog {
		public QuitDialog(JdctFrame frameIn,String title,String message)
		    {
			super(frameIn,title,true);

			setLayout(new BorderLayout(15, 15));
			add("Center",new Label(message,Label.CENTER));

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER,25,15));
			add("South",panel);

			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					QuitDialog.this.dispose();
					System.exit(0);
				} 
			}
			);
			Button cancelButton = new Button("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					QuitDialog.this.dispose();
				} 
			}
			);
			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					((Dialog)e.getSource()).dispose(); 					}
			}
			);
			panel.add(okButton);
			panel.add(cancelButton);
			pack();
			setVisible(true);
		}
	}

}
